﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [Item("Offspring Selection Sliding Window GP Analyzer", "Analyzer which moves a sliding window whenever the selection pressure exceeds a certain threshold.")]
  [StorableClass]
  public class OffspringSelectionSlidingWindowAnalyzer<T> : SlidingWindowAnalyzer<T>
    where T : class, IDataAnalysisProblemData {
    private const string SelectionPressureParameterName = "SelectionPressure";
    private const string SelectionPressureThresholdParameterName = "SelectionPressureThreshold";

    #region parameter properties
    public ILookupParameter<IntValue> SelectionPressureParameter {
      get { return (ILookupParameter<IntValue>)Parameters[SelectionPressureParameterName]; }
    }
    public IFixedValueParameter<IntValue> SelectionPressureThresholdParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters[SelectionPressureThresholdParameterName]; }
    }
    #endregion

    #region properties
    public IntValue SelectionPressureThreshold {
      get { return SelectionPressureThresholdParameter.Value; }
    }
    #endregion


    [StorableConstructor]
    protected OffspringSelectionSlidingWindowAnalyzer(bool deserializing) : base(deserializing) { }
    protected OffspringSelectionSlidingWindowAnalyzer(OffspringSelectionSlidingWindowAnalyzer<T> original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new OffspringSelectionSlidingWindowAnalyzer<T>(this, cloner);
    }

    public OffspringSelectionSlidingWindowAnalyzer() {
      Parameters.Add(new LookupParameter<IntValue>(SelectionPressureParameterName, ""));
      Parameters.Add(new FixedValueParameter<IntValue>(SelectionPressureThresholdParameterName, "", new IntValue(20)));
    }

    protected override bool CheckForUpdate() {
      var currentSelectionPressure = SelectionPressureParameter.ActualValue.Value;
      var selectionPressureThreshold = SelectionPressureThresholdParameter.Value.Value;

      return selectionPressureThreshold <= currentSelectionPressure;
    }

  }
}
